MODULE sbc_ice
   !!======================================================================
   !!                 ***  MODULE  sbc_ice  ***
   !!        parameter and  variables defined in memory in forced mode
   !!======================================================================
   !! History :  9.0  !  06-08  (G. Modec)  Surface module
   !!----------------------------------------------------------------------
#if defined key_lim3 || defined key_lim2
   !!----------------------------------------------------------------------
   !!   'key_lim2' or 'key_lim3' :             LIM 2.0 or 3.0 sea-ice model
   !!----------------------------------------------------------------------
   USE par_oce          ! ocean parameters
#if defined key_lim3
   USE par_ice          ! ice parameters
#endif

   IMPLICIT NONE
   PRIVATE
! variables used in forced and coupled mode
#if defined key_lim3 
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   qns_ice     !: non solar heat flux over ice  [W/m2]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   qsr_ice     !: solar heat flux over ice      [W/m2]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   dqns_ice    !: non solar heat flux sensibility over ice (LW+SEN+LA) [W/m2/K]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   alb_ice     !: albedo of ice
#else
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   qns_ice     !: non solar heat flux over ice  [W/m2]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   qsr_ice     !: solar heat flux over ice      [W/m2]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   dqns_ice    !: non solar heat flux sensibility over ice (LW+SEN+LA) [W/m2/K]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   alb_ice     !: albedo of ice
#endif

! Variables used only in forced mode
# if defined key_lim3 
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   qla_ice   !: latent flux over ice
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   dqla_ice  !: latent sensibility over ice
# else
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   qla_ice   !: latent flux over ice
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   dqla_ice  !: latent sensibility over ice
# endif

! Variables used only in coupled mode
#if defined key_lim3 
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   tn_ice      !: ice surface temperature       [K]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   tckice      !: ice thickness
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpl) ::   tcksnw      !: oce thickness
#else
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   tn_ice      !: ice surface temperature       [K]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   tckice      !: ice thickness
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   tcksnw      !: oce thickness
#endif
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   freeze      !: ice fraction
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   tprecip     !: total precipitation for ice   [Kg/m2/s]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   sprecip     !: solid precipitation ( - sublimation in coupled mode )          [Kg/m2/s]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   utaui_ice   !: u-stress over ice (I-point)   [N/m2]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   vtaui_ice   !: v-stress over ice (I-point)   [N/m2]
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   fr1_i0      !: 1st fraction of sol. rad.  which penetrate inside the ice cover
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   fr2_i0      !: 2nd fraction of sol. rad.  which penetrate inside the ice cover

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   rnfcpl       !: runoff
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj) ::   ocalving     !: calving into the ocean

#else
   !!----------------------------------------------------------------------
   !!   Empty module                                   NO LIM sea-ice model
   !!----------------------------------------------------------------------
#endif

   !!----------------------------------------------------------------------
   !!  OPA 9.0 , LOCEAN-IPSL (2005)
   !! $ Id: $ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
END MODULE sbc_ice
