MODULE sbccpl
   !!======================================================================
   !!                       ***  MODULE  sbccpl  ***
   !! Ocean forcing:  momentum, heat and freshwater coupled formulation
   !!=====================================================================
   !! History :  9.0   !  06-07  (R. Redler, N. Keenlyside, W. Park) 
   !!                            Original code split into flxmod & taumod
   !!            9.0   !  06-07  (G. Madec)  surface module
   !!----------------------------------------------------------------------
#if defined key_oasis3 || defined key_oasis4
   !!----------------------------------------------------------------------
   !!   'key_oasis3' or 'key_oasis4'   Coupled Ocean/Atmosphere formulation
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   namsbc_cpl   : coupled formulation namlist
   !!   sbc_cpl      : coupled formulation for the ocean surface boundary condition
   !!----------------------------------------------------------------------

   USE dom_oce         ! ocean space and time domain
   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE sbc_ice         ! Surface boundary condition: ice fields
#if defined key_lim3
   USE par_ice          ! ice parameters
#endif
   USE cpl_oasis3      ! OASIS3 coupling (to ECHAM5)
   USE geo2ocean
   USE restart
   USE in_out_manager  ! I/O manager
   USE iom             ! NetCDF library
   USE lib_mpp         ! distribued memory computing library
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)

   IMPLICIT NONE
   PRIVATE

   PUBLIC   sbc_cpl_snd       ! routine called by step.F90
   
   TYPE ::   FLD_CPL     ! Coupling Namelist field informations
      LOGICAL            ::   laction   ! 
      CHARACTER(len = 8) ::   clname      ! 
      CHARACTER(len = 1) ::   clgrid      !
      INTEGER            ::   nsgn      !
   END TYPE FLD_CPL
   

   TYPE(FLD_CPL):: sn_snd_fice






   REAL(wp), DIMENSION(jpi,jpj) :: qsr_mix




   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2006) 
   !! $ Id: $
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE sbc_cpl_init
 
      NAMELIST/namsbc_cpl_rcv/ ...


      !!---------------------------------------------------------------------




         REWIND( numnam )                    ! ... read in namlist namsbc_cpl_rcv
         READ  ( numnam, namsbc_cpl_rcv )
         
         !-------------------------------------
         !-------------------------------------
         ! Define the receive interface
         !-------------------------------------
         !-------------------------------------
         !
         ! Read restart of variables for coupling (needed to compute some values from the received data)


!!$   Probleme: definir comment on initialise freeze, alb_ice et tn_ice 
!!$             quand on n'a pas de restart (a nit000)


         CALL iom_get( numror, 'freeze' , freeze  )
         IF ( TRIM(cn_rcv_qsr) == 'mixed oce-ice' ) CALL iom_get( numror, 'alb_ice', alb_ice )
!!!!!
!!!!! +++ ERIC tu utilises tn_ice dans le calcule de Qns, c'est bien ca???
!!!!!
         IF ( TRIM(cn_rcv_qns) == 'mixed oce-ice' ) CALL iom_get( numror, 'tn_ice' , tn_ice  )





         ! default definitions of srcv
         nrcv = 0
         srcv(:)%cgrid = 'T'
         srcv(:)%nsgn = 1

         !-------------------------------------
         ! Qsr
         nrcv = nrcv + 1   ;   jprcv_qsroce = nrcv   ;   srcv(nrcv)%cname = 'O_QsrOce'
         nrcv = nrcv + 1   ;   jprcv_qsrice = nrcv   ;   srcv(nrcv)%cname = 'O_QsrIce'
         nrcv = nrcv + 1   ;   jprcv_qsrmix = nrcv   ;   srcv(nrcv)%cname = 'O_QsrMix'
         SELECT CASE (TRIM(cn_rcv_qsr))
         CASE( 'conservative'  )   ;   srcv( (/jprcv_qsrice, jprcv_qsrmix/) )%laction = .TRUE.
         CASE( 'oce and ice'   )   ;   srcv( (/jprcv_qsrice, jprcv_qsroce/) )%laction = .TRUE.
         CASE( 'mixed oce-ice' )   ;   srcv(                 jprcv_qsrmix   )%laction = .TRUE. 
         CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_qsr' )
         END SELECT

         !-------------------------------------
         ! Qns
         nrcv = nrcv + 1   ;   jprcv_qnsoce = nrcv   ;   srcv(nrcv)%cname = 'O_QnsOce'
         nrcv = nrcv + 1   ;   jprcv_qnsice = nrcv   ;   srcv(nrcv)%cname = 'O_QnsIce'
         nrcv = nrcv + 1   ;   jprcv_qnsmix = nrcv   ;   srcv(nrcv)%cname = 'O_QnsMix'
         SELECT CASE (TRIM(cn_rcv_qns))
         CASE( 'conservative'  )   ;   srcv( (/jprcv_qnsice, jprcv_qnsmix/) )%laction = .TRUE.
         CASE( 'oce and ice'   )   ;   srcv( (/jprcv_qnsice, jprcv_qnsoce/) )%laction = .TRUE.
         CASE( 'mixed oce-ice' )   ;   srcv(                 jprcv_qnsmix   )%laction = .TRUE. 
         CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_qns' )
         END SELECT

         !-------------------------------------
         ! emp, tprecip and sprecip
         nrcv = nrcv + 1   ;   jprcv_rain = nrcv   ;   srcv(nrcv)%cname = 'OIceRain'   ! Rain = liquid precipitation
         nrcv = nrcv + 1   ;   jprcv_snow = nrcv   ;   srcv(nrcv)%cname = 'OIceSnow'   
         nrcv = nrcv + 1   ;   jprcv_tevp = nrcv   ;   srcv(nrcv)%cname = 'OTotEvap'   ! total evaporation ( over oce + ice )
         nrcv = nrcv + 1   ;   jprcv_ievp = nrcv   ;   srcv(nrcv)%cname = 'OIceEvap'   ! evaporation iver ice (sublimation)
         nrcv = nrcv + 1   ;   jprcv_tpre = nrcv   ;   srcv(nrcv)%cname = 'OIPr-Sub'   ! Pr = liquid + solid precipitation
         nrcv = nrcv + 1   ;   jprcv_spre = nrcv   ;   srcv(nrcv)%cname = 'OISn-Sub'   ! Sub = Sublimation = Evap over ice
         nrcv = nrcv + 1   ;   jprcv_oemp = nrcv   ;   srcv(nrcv)%cname = 'OOEv-OPr'   ! 
         SELECT CASE (TRIM(cn_rcv_emp))
         CASE( 'conservative'  )   ;   srcv( (/jprcv_rain, jprcv_snow, jprcv_ievp, jprcv_tevp/) )%laction = .TRUE.
         CASE( 'oce and ice'   )   ;   srcv( (/            jprcv_tpre, jprcv_spre, jprcv_oemp/) )%laction = .TRUE.
         CASE( 'mixed oce-ice' )   ;   srcv( (/jprcv_rain,             jprcv_spre, jprcv_tevp/) )%laction = .TRUE. 
         CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_emp' )
         END SELECT

         !-------------------------------------
         ! wind stress : utau, vtau, utaui_ice, vtaui_ice 
         ! oce stress
         nrcv = nrcv + 1   ;   jprcv_otx1 = nrcv   ;   srcv(nrcv)%cname = 'O_OTaux1' ! oce tau 1st component on 1st grid
         nrcv = nrcv + 1   ;   jprcv_oty1 = nrcv   ;   srcv(nrcv)%cname = 'O_OTauy1' ! oce tau 2nd component on 1st grid
         nrcv = nrcv + 1   ;   jprcv_otz1 = nrcv   ;   srcv(nrcv)%cname = 'O_OTauz1' ! oce tau 3rd component on 1st grid
         nrcv = nrcv + 1   ;   jprcv_otx2 = nrcv   ;   srcv(nrcv)%cname = 'O_OTaux2' ! oce tau 1st component on 2nd grid
         nrcv = nrcv + 1   ;   jprcv_oty2 = nrcv   ;   srcv(nrcv)%cname = 'O_OTauy2' ! oce tau 2nd component on 2nd grid
         nrcv = nrcv + 1   ;   jprcv_otz2 = nrcv   ;   srcv(nrcv)%cname = 'O_OTauz2' ! oce tau 3rd component on 2nd grid
         ! ice stress
         nrcv = nrcv + 1   ;   jprcv_itx1 = nrcv   ;   srcv(nrcv)%cname = 'O_ITaux1' ! ice tau 1st component on 1st grid
         nrcv = nrcv + 1   ;   jprcv_ity1 = nrcv   ;   srcv(nrcv)%cname = 'O_ITauy1' ! ice tau 2nd component on 1st grid
         nrcv = nrcv + 1   ;   jprcv_itz1 = nrcv   ;   srcv(nrcv)%cname = 'O_ITauz1' ! ice tau 3rd component on 1st grid
         nrcv = nrcv + 1   ;   jprcv_itx2 = nrcv   ;   srcv(nrcv)%cname = 'O_ITaux2' ! ice tau 1st component on 2nd grid
         nrcv = nrcv + 1   ;   jprcv_ity2 = nrcv   ;   srcv(nrcv)%cname = 'O_ITauy2' ! ice tau 2nd component on 2nd grid
         nrcv = nrcv + 1   ;   jprcv_itz2 = nrcv   ;   srcv(nrcv)%cname = 'O_ITauz2' ! ice tau 3rd component on 2nd grid
         ! change default definition of srcv(:)%nsgn
         srcv(jprcv_otx1:jprcv_itz2)%nsgn = -1
         ! change default definition of srcv(:)%cgrid and srcv(:)%laction
         SELECT CASE (LEN_TRIM(cn_rcv_stress(4)))   !  'T' 'U,V' 'U,V,F' 'U,V,I' 'T,F' 'T,I' 'T,U,V'
         CASE( 1 )   ! 'T'
            srcv(jprcv_otx1:jprcv_itz2)%cgrid = cltmp(1)   ! all oce and ice components on the same unique grid
            srcv(jprcv_otx1:jprcv_otz1)%laction = .TRUE.   ! oce components on 1 grid 
            srcv(jprcv_itx1:jprcv_itz1)%laction = .TRUE.   ! ice components on 1 grid 
         CASE( 3 )   ! 'U,V' 'T,F' 'T,I'
            cltmp = cn_rcv_stress(4)
            SELECT CASE (cltmp(1))
            CASE( 'T' )   ! 'T,F' 'T,I'
                  srcv(jprcv_otx1:jprcv_otz2)%cgrid = cltmp(1)   ! oce and ice tau on 2 grids
                  srcv(jprcv_itx1:jprcv_itz2)%cgrid = cltmp(3)   ! but oce(ice) components on the same grid
                  srcv(jprcv_otx1:jprcv_otz1)%laction = .TRUE.   ! oce components on 1 grid
                  srcv(jprcv_itx1:jprcv_itz1)%laction = .TRUE.   ! ice components on 1 grid
            CASE( 'U' )   ! 'U,V'
               IF ( cltmp(3) := 'V' ) THEN CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
               srcv(jprcv_otx1:jprcv_otz1)%cgrid = cltmp(1)   ! oce(ice) components on 2 grids
               srcv(jprcv_otx2:jprcv_otz2)%cgrid = cltmp(3)
               srcv(jprcv_itx1:jprcv_itz1)%cgrid = cltmp(1)
               srcv(jprcv_itx2:jprcv_itz2)%cgrid = cltmp(3)
               srcv(jprcv_otx1:jprcv_itz2)%laction = .TRUE.   ! oce(ice) components on 2 grids
            CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
            END SELECT
         CASE( 5 )   ! 'U,V,F' 'U,V,I' 'T,U,V'
            cltmp = cn_rcv_stress(4)
            SELECT CASE (cltmp(1))
            CASE( 'T' )   ! 'T,U,V'
                  srcv(jprcv_otx1:jprcv_otz2)%cgrid = cltmp(1)    ! oce components on 1 grid
                  srcv(jprcv_itx1:jprcv_itz1)%cgrid = cltmp(3)    ! ice components on 2 grids
                  srcv(jprcv_itx2:jprcv_itz2)%cgrid = cltmp(5)
                  srcv(jprcv_otx1:jprcv_otz1)%laction = .TRUE.    ! oce components on 1 grid
                  srcv(jprcv_itx1:jprcv_itz2)%laction = .TRUE.    ! ice components on 2 grids
            CASE( 'U' )   ! 'U,V,F' 'U,V,I' 
               IF ( cltmp(3) := 'V' ) THEN CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
               srcv(jprcv_otx1:jprcv_otz1)%cgrid = cltmp(1)    ! oce components on 2 grids
               srcv(jprcv_otx2:jprcv_otz2)%cgrid = cltmp(3)
               srcv(jprcv_itx1:jprcv_itz2)%cgrid = cltmp(5)    ! ice components on 1 grid
               srcv(jprcv_otx1:jprcv_otz2)%laction = .TRUE.    ! oce components on 2 grids 
               srcv(jprcv_itx1:jprcv_itz1)%laction = .TRUE.    ! ice components on 1 grid 
            CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
            END SELECT
         CASE default   ;   CALL ctl_stop( 'sbc_cpl_init: wrong definition of cn_rcv_stress(4)' )
         END SELECT
         ! force .FALSE. to 3rd component for spherical coodinates
         IF ( TRIM(cn_rcv_stress(2)) == 'spherical' ) srcv((/jprcv_otz1, jprcv_otz2, jprcv_itz1, jprcv_itz2/))%laction = .FALSE. 
         ! force .FALSE. to ice components if not 'oce and ice'
         IF ( TRIM(cn_rcv_stress(1)) /= 'oce and ice' )   srcv(jprcv_itx1:jprcv_itz2)%laction = .FALSE. 

         !-------------------------------------
         ! 10 m wind speed
         nrcv = nrcv + 1   ;   jprcv_w10m = nrcv   ;   srcv(nrcv)%cname = 'O_Wind10'
         IF ( TRIM(cn_rcv_w10m) == 'coupled' ) srcv(jprcv_w10m)%laction = .TRUE.
         ! +++ ---> A brancher et a blinder dans tke  si TRIM(cn_rcv_w10m) == 'none'

         !-------------------------------------
         ! d(Qns)/d(T) 
         nrcv = nrcv + 1   ;   jprcv_dqnsdt = nrcv   ;   srcv(nrcv)%cname = 'O_dQnsdT'
         IF ( TRIM(cn_rcv_dqnsdt) == 'coupled' ) srcv(jprcv_dqnsdt)%laction = .TRUE.

         !-------------------------------------
         ! Runoff
         nrcv = nrcv + 1   ;   jprcv_rnf = nrcv   ;   srcv(nrcv)%cname = 'O_Runoff'
         IF ( TRIM(cn_rcv_rnf) /= 'climato' ) srcv(jprcv_rnf)%laction = .TRUE.
         ! +++ ---> A brancher  

         !-------------------------------------
         ! Calving
         nrcv = nrcv + 1   ;   jprcv_cal = nrcv   ;   srcv(nrcv)%cname = 'OCalving'
         IF ( TRIM(cn_rcv_cal) == 'coupled' ) srcv(jprcv_cal)%laction = .TRUE.
         ! +++ ---> A brancher  

         !-------------------------------------
         ! fraction of net shortwave radiation which is not absorbed in the 
         ! thin surface layer and penetrates inside the ice cover 
         !  ( Maykut and Untersteiner, 1971 ; Elbert anbd Curry, 1993 )
         ! Since cloud cover catm not transmitted from atmosphere 
         ! ===> defined as constant value -> definition done in sbc_cpl_init
!!$      catm(:,:) = 0.
!!$      zcatm1(:,:) = 1.0 - catm(:,:)  !  fractional cloud cover
!!$      fr1_i0(:,:) = 0.18 * zcatm1(:,:) + 0.35 * catm(:,:) 
!!$      fr2_i0(:,:) = 0.82 * zcatm1(:,:) + 0.65 * catm(:,:)
         fr1_i0(:,:) = 0.18
         fr2_i0(:,:) = 0.82

         !
         !-------------------------------------
         !-------------------------------------
         ! Define the send interface
         !-------------------------------------
         !-------------------------------------
         !
         ! default definitions of nsnd
         nsnd = 0
         ssnd(:)%cgrid = 'T'
         ssnd(:)%nsgn = 1
         
         !-------------------------------------
         ! Ice fraction
         nsnd = nsnd + 1   ;   jpsnd_fice = nsnd   ;   ssnd(nsnd)%cname = 'OIceFrac'
         ssnd(jpsnd_fice)%laction = .TRUE.
         
         !-------------------------------------
         ! T surf
         nsnd = nsnd + 1   ;   jpsnd_toce = nsnd   ;   ssnd(nsnd)%cname = 'O_SSTSST'
         nsnd = nsnd + 1   ;   jpsnd_tice = nsnd   ;   ssnd(nsnd)%cname = 'O_TepIce'
         nsnd = nsnd + 1   ;   jpsnd_tmix = nsnd   ;   ssnd(nsnd)%cname = 'O_TepMix'
         SELECT CASE (TRIM(cn_snd_temperature))
         CASE( 'oce only'             )   ;   ssnd(   jpsnd_toce               )%laction = .TRUE.
         CASE( 'weighted oce and ice' )   ;   ssnd( (/jpsnd_toce, jpsnd_tice/) )%laction = .TRUE.
         CASE( 'mixed oce-ice'        )   ;   ssnd(   jpsnd_tmix               )%laction = .TRUE.
         END SELECT

         !-------------------------------------
         ! Albedo
         nsnd = nsnd + 1   ;   jpsnd_albice = nsnd   ;   ssnd(nsnd)%cname = 'O_AlbIce'
         nsnd = nsnd + 1   ;   jpsnd_albmix = nsnd   ;   ssnd(nsnd)%cname = 'O_AlbMix'
         SELECT CASE (TRIM(cn_snd_albedo))
         CASE( 'none'          )       ! nothing to do
         CASE( 'weighted ice'  )   ;   ssnd(jpsnd_albice)%laction = .TRUE.
         CASE( 'mixed oce-ice' )   ;   ssnd(jpsnd_albmix)%laction = .TRUE.
         END SELECT
         
         !-------------------------------------
         ! Thickness
         nsnd = nsnd + 1   ;   jpsnd_tckice = nsnd   ;   ssnd(nsnd)%cname = 'O_IceTck'
         nsnd = nsnd + 1   ;   jpsnd_tcksnw = nsnd   ;   ssnd(nsnd)%cname = 'O_SnwTck'
         IF ( TRIM(cn_snd_thickness) == 'weighted ice and snow' ) ssnd( (/jpsnd_tckice, jpsnd_tcksnw/) )%laction = .TRUE.
         
         !-------------------------------------
         ! Surface current
         nsnd = nsnd + 1   ;   jpsnd_uoce = nsnd   ;   ssnd(nsnd)%cname = 'O_UN_Oce'
         nsnd = nsnd + 1   ;   jpsnd_voce = nsnd   ;   ssnd(nsnd)%cname = 'O_VN_OcE'
         nsnd = nsnd + 1   ;   jpsnd_uice = nsnd   ;   ssnd(nsnd)%cname = 'O_UN_Ice'
         nsnd = nsnd + 1   ;   jpsnd_vice = nsnd   ;   ssnd(nsnd)%cname = 'O_VN_IcE'
         nsnd = nsnd + 1   ;   jpsnd_umix = nsnd   ;   ssnd(nsnd)%cname = 'O_UN_Mix'
         nsnd = nsnd + 1   ;   jpsnd_vmix = nsnd   ;   ssnd(nsnd)%cname = 'O_VN_Mix'
         ssnd(jpsnd_uoce:jpsnd_vmix)%nsgn = -1
         SELECT CASE (TRIM(cn_snd_current(1)))
         CASE( 'none'                 )       ! nothing to do
         CASE( 'oce only'             )   ;   ssnd( (/jpsnd_uoce, jpsnd_voce                        /) )%laction = .TRUE.
         CASE( 'weighted oce and ice' )   ;   ssnd( (/jpsnd_uoce, jpsnd_voce, jpsnd_uice, jpsnd_vice/) )%laction = .TRUE.
         CASE( 'mixed oce-ice'        )   ;   ssnd( (/jpsnd_umix, jpsnd_vmix                        /) )%laction = .TRUE.
         END SELECT
         
         !
         !-------------------------------------
         !-------------------------------------
         CALL cpl_prism_define
         !-------------------------------------
         !-------------------------------------
         !

   END SUBROUTINE sbc_cpl_init



   SUBROUTINE sbc_cpl_rcv( kt )

      IF( kt == nit000 )   CALL sbc_cpl_init

      isec = ( kt - nit000 ) * NINT(rdttra(1))        ! date of exchanges

      !-------------------------------------
      ! Qsr : we must get qsr and qsr_ice
      IF ( srcv(jprcv_qsroce)%laction ) CALL cpl_prism_rcv( jprcv_qsroce, isec, qsr    , info ) 
      IF ( srcv(jprcv_qsrice)%laction ) CALL cpl_prism_rcv( jprcv_qsrice, isec, qsr_ice, info ) 
      IF ( srcv(jprcv_qsrmix)%laction ) CALL cpl_prism_rcv( jprcv_qsrmix, isec, qsr_mix, info )        
      SELECT CASE (TRIM(cn_rcv_qsr))
      CASE( 'conservative' )
         qsr(:,:) = ( qsr_mix(:,:) - freeze(:,:) * qsr_ice(:,:) ) / (1. - freeze(:,:))
      CASE( 'oce and ice' )
         ! nothing to do
      CASE( 'mixed oce-ice' )
         ztmp(:,:) = qsr_mix(:,:) / (1. - ( 0.065*(1. - freeze(:,:)) + freeze(:,:)*alb_ice(:,:)) )
         qsr_ice(:,:) = ztmp(:,:) * (1. - alb_ice(:,:))
         qsr    (:,:) = ztmp(:,:) * (1. - 0.065)
      END SELECT

      !-------------------------------------
      ! Qns : we must get qns and qns_ice
      IF ( srcv(jprcv_qnsoce)%laction ) CALL cpl_prism_rcv( jprcv_qnsoce, isec, qns    , info ) 
      IF ( srcv(jprcv_qnsice)%laction ) CALL cpl_prism_rcv( jprcv_qnsice, isec, qns_ice, info ) 
      IF ( srcv(jprcv_qnsmix)%laction ) CALL cpl_prism_rcv( jprcv_qnsmix, isec, qns_mix, info )        
      SELECT CASE (TRIM(cn_rcv_qns))
      CASE( 'conservative' )
         qns(:,:) = ( qns_mix(:,:) - freeze(:,:) * qns_ice(:,:) ) / (1. - freeze(:,:))
      CASE( 'oce and ice' )
         ! nothing to do
      CASE( 'mixed oce-ice' )
!!!!!
!!!!! +++ ERIC il faut que tu mettes les bonnes formules...
!!!!!
!!$         qns_ice(:,:) = ...
!!$         qns    (:,:) = ...
      END SELECT

      !-------------------------------------
      ! Precipitations and Evaporation: we must get emp tprecip and sprecip
      ! sprecip = snow_ice - evap_ice
      ! tprecip = ( rain_ice + snow_ice ) - evap_ice
      ! emp     = emp_oce = evap_oce - ( rain_oce + snow_oce ) ... runoff??? ... calving???
      IF ( srcv(jprcv_snow)%laction )   CALL cpl_prism_rcv( jprcv_snow, isec, zsnow  , info ) ! snow
      IF ( srcv(jprcv_rain)%laction )   CALL cpl_prism_rcv( jprcv_rain, isec, zrain  , info ) ! Rain = liquid precipitation
      IF ( srcv(jprcv_tevp)%laction )   CALL cpl_prism_rcv( jprcv_tevp, isec, ztevp  , info ) ! total evaporation (over oce + ice)
      IF ( srcv(jprcv_ievp)%laction )   CALL cpl_prism_rcv( jprcv_ievp, isec, zievp  , info ) ! evaporation over ice (sublimation)
      IF ( srcv(jprcv_tpre)%laction )   CALL cpl_prism_rcv( jprcv_tpre, isec, tprecip, info ) ! see above
      IF ( srcv(jprcv_spre)%laction )   CALL cpl_prism_rcv( jprcv_spre, isec, sprecip, info ) ! see above
      IF ( srcv(jprcv_oemp)%laction )   CALL cpl_prism_rcv( jprcv_oemp, isec, emp    , info ) ! see above
      SELECT CASE (TRIM(cn_rcv_emp))
      CASE( 'conservative' )
         sprecip(:,:) = zsnow(:,:) - zievp(:,:)
         tprecip(:,:) = zrain(:,:) + sprecip(,:)
         emp(:,:) = ( ztevp(:,:) - zievp(:,:)*(1. - freeze(:,:)) )/freeze(:,:) - tprecip(:,:)
      CASE( 'oce and ice' )
         ! nothing to do
      CASE( 'mixed oce-ice' )
         tprecip(:,:) = zrain(:,:) + sprecip(:,:)
         emp(:,:) = ztevp(:,:) - ( tprecip(:,:) + sprecip(:,:) )
      END SELECT
      
      !-------------------------------------
      ! wind stress : we must get utau, vtau, utaui_ice, vtaui_ice 
      ! oce stress
      IF ( srcv(jprcv_otx1)%laction )   CALL cpl_prism_rcv( jprcv_otx1, isec, zotx1, info ) ! oce tau 1st component on 1st grid
      IF ( srcv(jprcv_oty1)%laction )   CALL cpl_prism_rcv( jprcv_oty1, isec, zoty1, info ) ! oce tau 2nd component on 1st grid
      IF ( srcv(jprcv_otz1)%laction )   CALL cpl_prism_rcv( jprcv_otz1, isec, zotz1, info ) ! oce tau 3rd component on 1st grid
      IF ( srcv(jprcv_otx2)%laction )   CALL cpl_prism_rcv( jprcv_otx2, isec, zotx2, info ) ! oce tau 1st component on 2nd grid
      IF ( srcv(jprcv_oty2)%laction )   CALL cpl_prism_rcv( jprcv_oty2, isec, zoty2, info ) ! oce tau 2nd component on 2nd grid
      IF ( srcv(jprcv_otz2)%laction )   CALL cpl_prism_rcv( jprcv_otz2, isec, zotz2, info ) ! oce tau 3rd component on 2nd grid
      ! ice stress
      IF ( srcv(jprcv_itx1)%laction )   CALL cpl_prism_rcv( jprcv_itx1, isec, zitx1, info ) ! ice tau 1st component on 1st grid
      IF ( srcv(jprcv_ity1)%laction )   CALL cpl_prism_rcv( jprcv_ity1, isec, zity1, info ) ! ice tau 2nd component on 1st grid
      IF ( srcv(jprcv_itz1)%laction )   CALL cpl_prism_rcv( jprcv_itz1, isec, zitz1, info ) ! ice tau 3rd component on 1st grid
      IF ( srcv(jprcv_itx2)%laction )   CALL cpl_prism_rcv( jprcv_itx2, isec, zitx2, info ) ! ice tau 1st component on 2nd grid
      IF ( srcv(jprcv_ity2)%laction )   CALL cpl_prism_rcv( jprcv_ity2, isec, zity2, info ) ! ice tau 2nd component on 2nd grid
      IF ( srcv(jprcv_itz2)%laction )   CALL cpl_prism_rcv( jprcv_itz2, isec, zitz2, info ) ! ice tau 3rd component on 2nd grid
      ! cartesian to spherical coordinates -> 3 components to 2 components
      IF ( TRIM(cn_rcv_stress(2)) == 'cartesian' ) THEN 
         ! wind stress over ocean
         SELECT CASE (srcv(jprcv_otx1)%cgrid) 
         CASE( 'T' ) 
            CALL geo2oce ( zotx1, zoty1, zotz1, 'T', glamt, gphit, ztmpx1, ztmpy1 ) ! 1st and 2nd components on the same grid
         CASE( 'F' )
            CALL geo2oce ( zotx1, zoty1, zotz1, 'F', glamf, gphif, ztmpx1, ztmpy1 ) ! 1st and 2nd components on the same grid
         CASE( 'U' )
            CALL geo2oce ( zotx1, zoty1, zotz1, 'U', glamu, gphiu, ztmpx1, ztmpy1 ) ! 1st and 2nd components on the 1st grid
            CALL geo2oce ( zotx2, zoty2, zotz2, 'V', glamv, gphiv, ztmpx2, ztmpy2 ) ! 1st and 2nd components on the 2nd grid
            zotx2(:,:) = ztmpx2(:,:)   ! overwrite 1st component on the 2nd grid
            zoty2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
         END SELECT
         zotx1(:,:) = ztmpx1(:,:)   ! overwrite 1st component on the 1st grid
         zoty1(:,:) = ztmpy1(:,:)   ! overwrite 2nd component on the 1st grid
         ! wind stress over ice
         IF ( srcv(jprcv_itx1)%laction ) THEN 
            SELECT CASE (srcv(jprcv_itx1)%cgrid) 
            CASE( 'T' ) 
               CALL geo2oce ( zitx1, zity1, zitz1, 'T', glamt, gphit, ztmpx1, ztmpy1 ) ! 1st and 2nd comp. on the same grid
            CASE( 'F' )
               CALL geo2oce ( zitx1, zity1, zitz1, 'F', glamf, gphif, ztmpx1, ztmpy1 ) ! 1st and 2nd comp. on the same grid
            CASE( 'U' )
               CALL geo2oce ( zitx1, zity1, zitz1, 'U', glamu, gphiu, ztmpx1, ztmpy1 ) ! 1st and 2nd comp. on the 1st grid
               CALL geo2oce ( zitx2, zity2, zitz2, 'V', glamv, gphiv, ztmpx2, ztmpy2 ) ! 1st and 2nd comp. on the 2nd grid
               zitx2(:,:) = ztmpx2(:,:)   ! overwrite 1st comp. on the 2nd grid
               zity2(:,:) = ztmpy2(:,:)   ! overwrite 2nd comp. on the 2nd grid
            END SELECT
            zitx1(:,:) = ztmpx1(:,:)   ! overwrite 1st comp. on the 1st grid
            zity1(:,:) = ztmpy1(:,:)   ! overwrite 2nd comp. on the 2nd grid
         ENDIF
      ENDIF

      ! 'eastward-northward' to 'local grid' axes -> totate the components
      IF ( TRIM(cn_rcv_stress(3)) == 'eastward-northward' ) THEN                        ! Oce component
         call rot_rep( zotx1, zoty1, srcv(jprcv_otx1)%cgrid, 'en->i', ztmpx1 )      ! 1st component on the 1st grid
         zotx1(:,:) = ztmpx1(:,:)      ! overwrite 1st component on the 1st grid
         IF ( srcv(jprcv_otx2)%laction ) then 
            call rot_rep( zotx2, zoty2, srcv(jprcv_otx2)%cgrid, 'en->j', ztmpy2 )   ! 2nd component on the 2nd grid
            zoty2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
         ELSE
            call rot_rep( zotx1, zoty1, srcv(jprcv_otx1)%cgrid, 'en->j', ztmpy1 )   ! 2nd component on the 1st grid
            zoty1(:,:) = ztmpy1(:,:)   ! overwrite 2nd component on the 1st grid
         ENDIF
         IF ( srcv(jprcv_itx1)%laction ) THEN                                            ! Ice component
            call rot_rep( zitx1, zity1, srcv(jprcv_itx1)%cgrid, 'en->i', ztmpx1 )      ! 1st component on the 1st grid
            zitx1(:,:) = ztmpx1(:,:)      ! overwrite 1st component on the 1st grid
            IF ( srcv(jprcv_itx2)%laction ) THEN
               call rot_rep( zitx2, zity2, srcv(jprcv_itx2)%cgrid, 'en->j', ztmpy2 )   ! 2nd component on the 2nd grid
               zity2(:,:) = ztmpy2(:,:)   ! overwrite 2nd component on the 2nd grid
            ELSE
               call rot_rep( zitx1, zity1, srcv(jprcv_itx1)%cgrid, 'en->j', ztmpy1 )   ! 2nd component on the 1st grid
               zity1(:,:) = ztmpy1(:,:)   ! overwrite 2nd component on the 1st grid
            ENDIF
         ENDIF
      ENDIF

      ! oce stress must be on U,V grids
      IF ( srcv(jprcv_otx1)%cgrid == 'T' ) THEN
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
              utau(ji,jj) = 0.5 * ( zotx1(ji,jj) + zotx1(ji+1,jj  ) ) ! T -> U grid
              vtau(ji,jj) = 0.5 * ( zoty1(ji,jj) + zoty1(ji  ,jj+1) ) ! T -> V grid
            END DO
         END DO
         CALL lbc_lnk( utau, 'U',  -1. )   ;   CALL lbc_lnk( vtau, 'V',  -1. )
      ELSE
         utau(:,:) = zotx1(:,:)
         vtau(:,:) = zoty2(:,:)
      ENDIF

      ! make sure we have stress over ice
      IF ( TRIM(cn_rcv_stress(1)) /= 'oce and ice' ) THEN 
         zitx1(:,:) = zotx1(:,:)                                                ! 1st component on the 1st grid
         IF ( srcv(jprcv_otx2)%laction ) THEN   ;   zity2(:,:) = zoty2(:,:)   ! 2nd component on the 2nd grid
         ELSE                                     ;   zity1(:,:) = zoty1(:,:)   ! 2nd component on the 1st grid
         ENDIF
         srcv(jprcv_itx1)%cgrid = srcv(jprcv_otx1)%cgrid   ! update grid of the ice component
      ENDIF
      
      ! ice stress must be on I grid
      SELECT CASE ( srcv(jprcv_itx1)%cgrid )
      CASE( 'U' )
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               utaui_ice(ji,jj) = 0.5 * ( zitx1(ji-1,jj  ) + zitx1(ji-1,jj-1) ) ! U -> I grid
               vtaui_ice(ji,jj) = 0.5 * ( zity2(ji  ,jj-1) + zity2(ji-1,jj-1) ) ! V -> I grid
            END DO
         END DO
         CALL lbc_lnk( utaui_ice, 'I',  -1. )   ;   CALL lbc_lnk( vtaui_ice, 'I',  -1. )
      CASE( 'F' )
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               utaui_ice(ji,jj) = zitx1(ji-1,jj-1) ! F -> I grid
               vtaui_ice(ji,jj) = zity1(ji-1,jj-1) ! F -> I grid
            END DO
         END DO
         CALL lbc_lnk( utaui_ice, 'I',  -1. )   ;   CALL lbc_lnk( vtaui_ice, 'I',  -1. )
      CASE( 'T' )
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               utaui_ice(ji,jj) = 0.25 * ( zitx1(ji,jj) + zitx1(ji-1,jj) + zitx1(ji,jj-1) + zitx1(ji-1,jj-1) ) ! T -> I grid
               vtaui_ice(ji,jj) = 0.25 * ( zity1(ji,jj) + zity1(ji-1,jj) + zity1(ji,jj-1) + zity1(ji-1,jj-1) ) ! T -> I grid
            END DO
         END DO
         CALL lbc_lnk( utaui_ice, 'I',  -1. )   ;   CALL lbc_lnk( vtaui_ice, 'I',  -1. )
      CASE( 'I' )
         utaui_ice(:,:) = zitx1(:,:)
         vtaui_ice(:,:) = zity1(:,:)
      END SELECT
      
      !-------------------------------------
      ! 10 m wind speed
      ! +++ ---> blinder dans tke  si TRIM(cn_rcv_w10m) == 'none'
!!$     +++ IF ( srcv(jprcv_w10m  )%laction )   CALL cpl_prism_rcv( jprcv_dqnsdt, isec, wind10, info )
      
      !-------------------------------------
      ! d(Qns)/d(T) 
      IF ( srcv(jprcv_dqnsdt)%laction )   CALL cpl_prism_rcv( jprcv_dqnsdt, isec, dqns_ice, info )
      
      !-------------------------------------
      ! Runoff
      IF ( srcv(jprcv_rnf   )%laction )   CALL cpl_prism_rcv( jprcv_rnf   , isec, rnfcpl, info )
      
      !-------------------------------------
      ! Calving
      IF ( srcv(jprcv_cal   )%laction )   CALL cpl_prism_rcv( jprcv_cal   , isec, ocalving, info )
      
      !  fraction of net shortwave radiation which is not absorbed in the 
      !  thin surface layer and penetrates inside the ice cover 
      !  ( Maykut and Untersteiner, 1971 ; Elbert anbd Curry, 1993 )
      !------------------------------------------------------------------
      ! Since cloud cover catm not transmitted from atmosphere
      ! ===> defined as constant value -> definition done in sbc_cpl_init

   END SUBROUTINE sbc_cpl_rcv
   
   SUBROUTINE sbc_cpl_snd( kt )
      
      isec = ( kt - nit000 ) * NINT(rdttra(1))        ! date of exxhanges

      !-------------------------------------
      ! Ice fraction
      IF ( ssnd(jpsnd_fice)%laction ) CALL cpl_prism_snd( jpsnd_fice, isec, freeze, info )

      !-------------------------------------
      ! T surf
      ztmp(:,:) = tn(:,:,1) + rt0
      SELECT CASE (TRIM(cn_snd_temperature))
      CASE( 'oce only'             )       ! nothing to do
      CASE( 'weighted oce and ice' )   ;   ztmp = ztmp(:,:) * (1. - freeze(:,:))
      CASE( 'mixed oce-ice'        )   ;   ztmp = ztmp(:,:) * (1. - freeze(:,:)) + tn_ice(:,:)*freeze(:,:)
      END SELECT
      IF ( ssnd(jpsnd_toce)%laction ) CALL cpl_prism_snd( jpsnd_toce, isec, ztmp, info )
      IF ( ssnd(jpsnd_tice)%laction ) CALL cpl_prism_snd( jpsnd_tice, isec, tn_ice(:,:) * freeze(:,:), info )
      IF ( ssnd(jpsnd_tmix)%laction ) CALL cpl_prism_snd( jpsnd_tmix, isec, ztmp, info )
      
      !-------------------------------------
      ! Albedo
      IF ( ssnd(jpsnd_albice)%laction ) CALL cpl_prism_snd( jpsnd_albice, isec, alb_ice(:,:) * freeze(:,:), info )
      IF ( ssnd(jpsnd_albmix)%laction ) THEN 
!!!!! +++ ERIC   ztmp(:,:) = albedo de l'ocean a definir...
         CALL cpl_prism_snd( jpsnd_albmix, isec, ztmp(:,:) * (1. - freeze(:,:)) + alb_ice(:,:) * freeze(:,:), info )
      ENDIF
      
      !-------------------------------------
      ! Thickness
      IF ( ssnd(jpsnd_tckice)%laction ) CALL cpl_prism_snd( jpsnd_tckice, isec, tckice(:,:) * freeze(:,:), info )
      IF ( ssnd(jpsnd_tcksnw)%laction ) CALL cpl_prism_snd( jpsnd_tcksnw, isec, tcksnw(:,:) * freeze(:,:), info )


      
      !-------------------------------------
      ! Surface current

      +++ seb ecriture des restarts...


   END SUBROUTINE sbc_cpl_snd
   




#else
      !!----------------------------------------------------------------------
      !!   Dummy routine                              NO coupling
      !!----------------------------------------------------------------------

+++ a verifier ...


#endif

      !!======================================================================
   END MODULE sbccpl
